using System.Diagnostics;
using System.Net.Mime;
using System.Text;
using LuskiServer.Classes.TableDef;
using LuskiServer.Enums;
using LuskiServer.Interfaces;
using Microsoft.AspNetCore.Mvc;

namespace LuskiServer.Classes;

public static class EXT
{/*
    public static bool CanTokenRequest<TResult>(this ControllerBase Base, out long id, out TResult? result) where TResult : HTTPResponse, new()
    {
        if (Base.Request.Headers.ContainsKey("token"))
        {
            return ValidateToke(Base, out id, out result);
        }
        result = Base.ShowError<TResult>(ErrorCode.MissingToken);
        id = 0;
        return false;
    }
    */
    
    
    public static string GetNumberString(this ServerPermissions enu)
    {
       return ((long)(enu)).ToString();
    }
    
    public static bool CanTokenRequest(this ControllerBase Base, out long id, out IActionResult? result)
    {
        if (Base.Request.Headers.ContainsKey("token"))
        {
            return ValidateToke(Base, out id, out result);
        }
        result = Base.ShowError(ErrorCode.MissingToken, "You did not provide a token");
        id = 0;
        return false;
    }

    public static IActionResult ShowError(this ControllerBase Base, Exception Error, bool LogInDB = true)
    {
        if (LogInDB)
        {
            Tables.Logs.Insert(
                Logs.ID.CreateParameter(Luski.Snowflake.GenerateSnowflake(Luski.Config.ServerEpoch) .ID), 
                Logs.Type.CreateParameter(LogType.Error),
                Logs.Message.CreateParameter(Error.ToString()));
        }

        return Base.ShowError(ErrorCode.ServerError, Error.Message);
    }
    
    public static IActionResult ShowError(this ControllerBase Base, ErrorCode code, string Error)
    {
        return Base.StatusCode(403, new HTTPResponse()
        {
            error = code,
            error_message = Error
        });
    }
    /*
    public static TReturn ShowError<TReturn>(this ControllerBase Base, ErrorCode code, Exception? Error, bool LogInDB = true) where TReturn : HTTPResponse, new()
    {
        if (Error is null) return new TReturn()
        {
            error = ErrorCode.ServerError,
            error_message = "Something Went wrong in the error handler."
        };

        try
        {
            StackTrace st = new(Error, true);
            StackFrame? frame = st.GetFrame(0);
            string? controler = Base.RouteData.Values["controller"]?.ToString();
            string Line = "'Unknown'";
            if (frame is not null) Line = frame.GetFileLineNumber().ToString();
            string msg = $"In '{controler}Controller.cs' on line '{Line}' through error '{Error.Message}' and ST'{st}' and ERROR:\n'{Error}' FRAM:\n '{frame}'";
            //if (LogInDB) Log(msg, LogType.Error);
            return Base.ShowError<TReturn>(code, Error.Message);
        }
        catch (Exception ex)
        {
            Console.WriteLine(ex);
            return new TReturn()
            {
                error = ErrorCode.ServerError,
                error_message = "Something Went wrong in the error handler"
            };
            
        }
    }
    
    public static TReturn ShowError<TReturn>(this ControllerBase Base, ErrorCode code, string? Error = null) where TReturn : HTTPResponse, new()
    {
        Base.Response.ContentType = "application/json";
        switch (code)
        {
            case ErrorCode.Forbidden or ErrorCode.InvalidHeader or ErrorCode.InvalidToken or ErrorCode.MissingToken or ErrorCode.MissingToken or ErrorCode.MissingHeader:
                Base.Response.StatusCode = StatusCodes.Status403Forbidden;
                break;
        }
        TReturn hTTPResponse = new()
        {
            error = code,
            error_message = Error
        };
        return hTTPResponse;
    }
    */
    private static bool CheckToken(string Token, ref long ID)
    {
        try
        {
            string id = Encoding.UTF8.GetString(Convert.FromBase64String(Token.Split('.')[0]));
            string? tok = Tables.Users.Read(Users.Token, Users.ID.CreateParameter(long.Parse(id)));
            if (!string.IsNullOrWhiteSpace(tok) && tok == Token)
            {
                ID = long.Parse(id);
                return true;
            }
            return false;
        }
        catch
        {
            return false;
        }
    }
    
    private static bool ValidateToke(ControllerBase Base, out long Id, out IActionResult? result)
    {
        long id = 0;
        if (CheckToken(Base.Request.Headers["token"].First()!, ref id))
        {
            result = null;
            Id = id;
            return true;
        }
        Id = id;
        result = Base.ShowError(ErrorCode.InvalidToken, (string)null!);
        return false;
    }
}