using System.Net.Mime;
using System.Security.Cryptography;
using System.Text;
using Asp.Versioning;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using LuskiServer.Enums;
using Microsoft.AspNetCore.Mvc;

namespace LuskiServer.Controllers.v1;

[ApiVersion(1)]
[ApiController]
public class CreateAccountController : ControllerBase
{
    /// <summary>
    /// Make a post request to this endpoint for registering an account on the server.
    /// </summary>
    /// <param name="UsernameRaw">Encrypted email for account</param>
    /// <param name="PasRaw">Hashed Password for account</param>
    /// <param name="DisplayName">Plain text Username</param>
    /// <param name="KeyRaw">Plain text RSA key for server to respond with</param>
    /// <param name="Body">Raw Picture data for profile</param>
    /// <returns></returns>
    [HttpPost]
    [DisableRequestSizeLimit]
    [Consumes(MediaTypeNames.Image.Jpeg, MediaTypeNames.Image.Gif, MediaTypeNames.Image.Tiff, "image/png")]
    [Produces(MediaTypeNames.Application.Json)]
    [ProducesResponseType(typeof(Login), StatusCodes.Status201Created)]
    [ProducesResponseType(typeof(Login), StatusCodes.Status403Forbidden)]
    [Route(Luski.Info.Routes.Default.Base)]
    public IActionResult Post([FromBody]byte[] Body, [FromHeader(Name = "username")]string? UsernameRaw, [FromHeader(Name = "password")]string? PasRaw, [FromHeader(Name = "displayname")]string? DisplayName, [FromHeader(Name = "key")]string? KeyRaw)
    {
        try
        {
            PictureType pfp;
            byte[] PasBytes, Username;
            byte[] salt = new byte[100];
            try
            {
                using (RandomNumberGenerator provider = RandomNumberGenerator.Create())
                {
                    provider.GetBytes(salt);
                }
                try
                {
                    PasBytes = Luski.Encryption.RemotePasswordEncrypt(Luski.Encryption._encoder.GetString(Luski.Encryption.Decrypt(Convert.FromBase64String(PasRaw))), salt);
                    Username = Luski.Encryption.Decrypt(Convert.FromBase64String(UsernameRaw));

                    try
                    {
                        byte[] g = Luski.Encryption.Encrypt("Test data to send to client", KeyRaw);
                    }
                    catch
                    {
                        return this.ShowError(ErrorCode.InvalidHeader, "The public keys you gave cant be used to encrypt data");
                    }
                }
                catch
                {
                    return this.ShowError(ErrorCode.InvalidHeader, "Make sure your login is encrypted with the server provided public key");
                }
            }
            catch (Exception ex)
            {
                if (ex.Message.Contains("not found")) return this.ShowError(ErrorCode.MissingHeader, "Missing login infermation");
                return this.ShowError(ex);
            }

            if (CheckUsername(Username))
            {
                int num = new Random().Next(1000, 1000000000);
                int num2 = new Random().Next(1000, 1000000000);
                Luski.Snowflake id = Luski.Snowflake.GenerateSnowflake(Luski.Config.ServerEpoch);
                byte[] ID = Encoding.UTF8.GetBytes(id.ID.ToString());
                byte[] Timestamp = Encoding.UTF8.GetBytes(DateTime.UtcNow.ToString());
                byte[] Number = Encoding.UTF8.GetBytes(num.ToString());
                byte[] Number2 = Encoding.UTF8.GetBytes(num2.ToString());
                string Token = $"{Convert.ToBase64String(ID)}.{Convert.ToBase64String(Timestamp)}.{Convert.ToBase64String(Number)}.{Convert.ToBase64String(Number2)}";

                pfp = GetProfilePictureType(Encoding.UTF8.GetString(Body).ToUpper());
                
                Tables.Users.Insert(
                    Users.ID.CreateParameter(id.ID),
                    Users.DisplayName.CreateParameter(DisplayName!.Replace("'", "\'")),
                    Users.SelectedChannel.CreateParameter(0), //TODO set to default
                    Users.Status.CreateParameter(Status.Offline),
                    Users.PictureType.CreateParameter(pfp),
                    Users.Picture.CreateParameter(Body),
                    Users.Roles.CreateParameter(Array.Empty<long>()),
                    Users.Username.CreateParameter(Username),
                    Users.Password.CreateParameter(PasBytes),
                    Users.Salt.CreateParameter(salt),
                    Users.LoginToken.CreateParameter(Token),
                    Users.SessionKey.CreateParameter(KeyRaw),
                    Users.WSSTCP.CreateParameter(string.Empty),
                    Users.Token.CreateParameter(string.Empty),
                    Users.OfflineData.CreateParameter(Array.Empty<string>()),
                    Users.OffileKey.CreateParameter(string.Empty));
                Thread t = new(o => RegToken((string?)o));
                t.Start(Token);
                return StatusCode(201, new Login()
                {
                    login_token = Token
                });
            }
            return this.ShowError(ErrorCode.Forbidden, "That email is already being used");
        }
        catch (Exception ex)
        {
            return this.ShowError(ex);
        }
    }

    private static PictureType GetProfilePictureType(string headerCode)
    {
        if (headerCode.StartsWith("FFD8FFE0"))
        {
            return PictureType.jpeg;
        }
        else if (headerCode.StartsWith("49492A"))
        {
            return PictureType.tif;
        }
        else if (headerCode.StartsWith("424D"))
        {
            return PictureType.bmp;
        }
        else if (headerCode.StartsWith("GIF"))
        {
            return PictureType.gif;
        }
        else if (headerCode.Remove(0, 1).StartsWith("PNG"))
        {
            return PictureType.png;
        }
        else
        {
            return PictureType.png;
        }
    }

    /// <summary>
    /// Cheaks to see Username is free
    /// </summary>
    /// <param name="Username"></param>
    /// <returns>
    /// <para>true = Good</para>
    /// <para>false = Username Taken</para>
    /// </returns>
    private static bool CheckUsername(byte[] Username)
    {
        if (Tables.Users.TryRead(Users.Username, out _, Users.Username.CreateParameter(Username))) return false;
        return true;
    }

    private static void RegToken(string? token)
    {
        try
        {
            if (token == null) return;
            long id = long.Parse(Encoding.UTF8.GetString(Convert.FromBase64String(token.Split('.')[0])));
            Tables.Users.Update(Users.ID, id, Users.LoginToken.CreateParameter(token));
            Thread.Sleep(30000);
            Tables.Users.Update(Users.ID, id, Users.LoginToken.CreateParameter(string.Empty));
        }
        catch (Exception ex)
        {
            Console.WriteLine(ex.Message);
        }
    }
}