using System.Net.Mime;
using System.Text;
using Asp.Versioning;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using LuskiServer.Enums;
using Microsoft.AspNetCore.Mvc;

namespace LuskiServer.Controllers.v1;

[ApiVersion(1)]
[ApiController]
public class KeysController : ControllerBase
{
    /// <summary>
    /// Return a public RSA key to encrypt information directed for the server.
    /// </summary>
    /// <returns></returns>
    [HttpGet]
    [ProducesResponseType(StatusCodes.Status200OK)]
    [Produces(MediaTypeNames.Application.Xml)]
    [Route(Luski.Info.Routes.Default.Base + "/PublicKey")]
    public IActionResult PublicKey()
    {
        return File(Encoding.UTF8.GetBytes(Luski.Encryption.Keys.PublicKey), "application/xml");
    }
    
    /// <summary>
    /// Make a post request to this URL to set you offline data key.
    /// </summary>
    /// <param name="token">Your Luski token for the server</param>
    /// <param name="keyy">The key you want to set for when you go offline</param>
    /// <returns></returns>
    [HttpPost]
    [Consumes(MediaTypeNames.Application.Xml)]
    [Produces(MediaTypeNames.Application.Json)]
    [ProducesResponseType(StatusCodes.Status202Accepted)]
    [ProducesResponseType(typeof(HTTPResponse), StatusCodes.Status403Forbidden)]
    [Route(Luski.Info.Routes.Default.Base + "/SetOfflineKey")]
    public IActionResult SetOfflineKey([FromBody]byte[]? keyy, [FromHeader(Name = "token")]string? token)
    {
        try
        {
            string key = "";
            if (!this.CanTokenRequest(out long ID, out IActionResult? toc) && toc != null) return toc;
            string[]? data = Tables.Users.Read(Users.OfflineData, Users.ID.CreateParameter(ID));
            if (data is not null) return this.ShowError(ErrorCode.Forbidden, "you cant change your key untill you download your data");
            
            try
            {
                byte[] g = Luski.Encryption.Encrypt("Test data to send to client", key);
            }
            catch
            {
                
                return this.ShowError(ErrorCode.InvalidPostData, "The key you sent the server appears to be incorect");
            }
            Tables.Users.Update(Users.ID, ID,
                Users.OffileKey.CreateParameter(key));
            return StatusCode(202);
        }
        catch (Exception ex)
        {
            return this.ShowError(ex);
        }
    }
}